<?php

namespace User;

use Core\Config;
use Core\Db;
use Helper\Mail;
use PassGen;

class Confirmation
{
    /**
     * Для однотипных писем с подтверждением.
     * @param array $userinfo Данные из (User)->info.
     * @param string $value Новое значение
     * @param ?int $code Проверочный код
     * @param string $type Тип меняемого значения. С маленькой буквы, именительный падеж.
     * @return void
     */
    public static function byEmail(array $userinfo, string $type, string $value, ?int $code = null): void
    {
        $ip = UserIp::get();
        $date = date('d.m.y H:i');
        $https = Config::get('https');
        $support = Config::get('support');
        $activationLink = Config::get('https') . "/confirm.php?id={$userinfo['id']}&code=$code";
        $fulllogin = $userinfo['login'] . "[{$userinfo['level']}]";
        Mail::send(
            $userinfo['mail'],
            <<<HTML
        <html lang="ru">
        <head><title>Сменить $type</title></head>
        <body>
        $date<br>
        Кто-то с IP: $ip пытается сменить $type к персонажу $fulllogin.<br>
        Т.к. в анкете у этого персонажа указан email: {$userinfo['mail']}, то вы и получили это письмо.<br>
        login: {$userinfo['login']}<br>
        Новый $type: <span style="background-color: wheat; font-family: Consolas, monospace;">$value</span><br><br>
        Для того чтобы сменить $type, перейдите по ссылке:<br>
        $activationLink<br>
        <br>--<br>
        Бойцовский Клуб $https<br>
        Администрация Бойцовского Клуба: $support<br>
        P.S. Данное письмо сгенерировано автоматически, отвечать на него не нужно.
        </body>
        </html>
        HTML,
            "Смена $type у персонажа $fulllogin"
        );
    }

    public static function userRegistrationCodeByEmail(string $email, string $login): void
    {
        $code = PassGen::intCode(4);
        Db::sql('insert into secure_code (email, code, time) values (?,?,unix_timestamp())', [$email, $code]);
        Mail::send(
            $email,
            'Секретный Код: ' . $code,
            'Код подтверждения регистрации персонажа ' . $login
        );
    }

    public static function byCode(int $uid, int $code): string
    {
        $status = '';

        if ($uid <= 0 || $code <= 0) {
            return $status;
        }

        $codes = Db::getRow(
            'select * from emailconfirmation where id = ? and code = ? and (active = 1 or pass = 1 or email = 1)',
            [$uid, $code]
        );
        if (!$codes['id']) {
            $status = 'Ссылка устарела!';
        } elseif ($codes['active'] === 1) {
            Db::sql(
                'update users set emailconfirmation = 1, securetime = unix_timestamp() + 259200 where id = ?',
                [$codes['id']]
            );
            $status = "Подтверждение смены пароля/email через почту включено";
        } elseif ($codes['pass'] === 1) {
            Db::sql(
                'update users as u inner join emailconfirmation as e on u.id = e.id set u.pass = e.pa_em, u.securetime = unix_timestamp() + 259200 where u.id = ?',
                [$codes['id']]
            );
            $status = "Удачно сменили пароль<";
        } elseif ($codes['email'] === 1) {
            Db::sql(
                'update users as u inner join emailconfirmation as e on u.id = e.id set u.mail = e.pa_em, u.securetime = unix_timestamp() + 259200 where u.id = ?',
                [$codes['id']]
            );
            $status = "Удачно сменили email";
        }
        Db::sql('delete from emailconfirmation where id = ? and code = ?', [$_GET['id'], $_GET['code']]);
        if ($status) {
            $status = "<h3>$status</h3>";
        }
        return $status;
    }
}
